/* Copyright 2006, freshmowed software.  All rights reserved. */

/*
*  Each Scale represents a specific x-axis, ususally this is for a timeframe, like 1 week, 6 months, etc., but
*  can also be dynamic, like YTD, MTD, or "auto" (adjusting to data's min/max x-values).  The y-axis scaling
*  support is for multiple datasets.  So, for a given time frame, multiple charts can be stacked.
*/ 
SCALE_AUTO_KEY = "Auto";
SCALE_1_WEEK_KEY = "1w";
SCALE_1_MONTH_KEY = "1m";
SCALE_3_MONTH_KEY = "3m";
SCALE_6_MONTH_KEY = "6m";
SCALE_MTD_KEY = "MTD";
SCALE_YTD_KEY = "YTD";
SCALE_1_YEAR_KEY = "1y";

SCALE_MIN_X_SPAN = 1;
SCALE_BIGNUM = 9999999;


Scale = function(aKey, aLabel)
{
  if ( Scale.directory == null ) Scale.directory = new Array();
  if ( Scale.list == null ) Scale.list = new Array();
  if ( Scale.current == null ) Scale.current = this;
  
  if ( aKey == null ) return;
  
  this.key = aKey;
  Scale.directory[this.key] = this;
  Scale.list.push(this);
  
  
  this.numBuckets = (this.key == SCALE_AUTO_KEY) ? SCALE_MIN_X_SPAN : null;
  this.numBucketsByTrackerIndex = new Array();
  this.xMin = SCALE_BIGNUM;
  this.xMax = -SCALE_BIGNUM;
  this.xMinArray = new Array();
  this.xMinArray[0] = SCALE_BIGNUM;
  this.xMinArray[1] = SCALE_BIGNUM;
  this.xMinArray[2] = SCALE_BIGNUM;
  this.xMaxArray = new Array();
  this.xMaxArray[0] = -SCALE_BIGNUM;
  this.xMaxArray[1] = -SCALE_BIGNUM;
  this.xMaxArray[2] = -SCALE_BIGNUM;
  this.yMin = new Array();
  this.yMax = new Array();
  this.label = (aLabel == null) ? aKey : aLabel;
}


Scale.prototype = 
{
    getNumBuckets: function(anIndex)
    {
        var result = (anIndex == null) ? this.numBuckets : this.numBucketsByTrackerIndex[anIndex];

        if ( result < SCALE_MIN_X_SPAN ) result = SCALE_MIN_X_SPAN;
        
        return result;
    },
    
    setNumBuckets: function(numBuckets, anIndex)
    {
      if ( anIndex == null )
        this.numBuckets = numBuckets;
      else
        this.numBucketsByTrackerIndex[anIndex] = numBuckets;
    },
    
    reset: function()
    {
        this.numBuckets = (this.key == SCALE_AUTO_KEY) ? SCALE_MIN_X_SPAN : null;
        this.numBucketsByTrackerIndex = new Array();
        this.xMin = SCALE_BIGNUM;
        this.xMax = -SCALE_BIGNUM;
        this.xMinArray = new Array();
        this.xMinArray[0] = SCALE_BIGNUM;
        this.xMinArray[1] = SCALE_BIGNUM;
        this.xMinArray[2] = SCALE_BIGNUM;
        this.xMaxArray = new Array();
        this.xMaxArray[0] = -SCALE_BIGNUM;
        this.xMaxArray[1] = -SCALE_BIGNUM;
        this.xMaxArray[2] = -SCALE_BIGNUM;
        this.yMin = new Array();
        this.yMax = new Array();
    },
  
    adjustMinMax: function(someData, anIndex)
    {
        this.xMin[anIndex] = SCALE_BIGNUM;
        this.xMax[anIndex] = -SCALE_BIGNUM;
        this.yMin[anIndex] = SCALE_BIGNUM;
        this.yMax[anIndex] = -SCALE_BIGNUM;
        
        for ( i = 0; i < someData.length; i++)
        {
            if ( someData[i] == null || someData[i].xValue == null ) continue; 
            
            if ( someData[i].xValue != null && someData[i].yValue == null && someData[i].note == null ) continue;

          // if in a fixed range mode, filter out any points falling outside the range
          if ( this.key != SCALE_AUTO_KEY && this.xMax != -SCALE_BIGNUM && this.xMin != SCALE_BIGNUM &&
               (someData[i].xValue > this.xMax ||
                someData[i].xValue < this.xMin ) )
           {               
               continue;
           }
            
           this.adjustMinMaxForPoint(someData[i], anIndex);
        }
        
        if ( this.key == SCALE_AUTO_KEY )
        {
          this.setNumBuckets((this.getXMax(anIndex) - this.getXMin(anIndex)) + 1, anIndex);
        }
        //alert("Scale.adjMinMax: xMin: " + this.xMinArray[anIndex] + " xMax: " + this.xMaxArray[anIndex] + 
        //     " yMin[" + anIndex + "]: " + this.yMin[anIndex] + " yMax[" + anIndex + "]: " + this.yMax[anIndex]);
    },
    
  /* This method determines if the passed point holds a new min or max in
   * both the x and y axes.
   */
  adjustMinMaxForPoint: function(aPoint, anIndex)
  {
      if ( this.key == SCALE_AUTO_KEY )
      {
          if ( aPoint.xValue > this.xMaxArray[anIndex] ) this.xMaxArray[anIndex] = aPoint.xValue;
          if ( aPoint.xValue > this.xMax ) this.xMax = aPoint.xValue;
          if ( aPoint.xValue < this.xMinArray[anIndex] ) this.xMinArray[anIndex] = aPoint.xValue;
          if ( aPoint.xValue < this.xMin ) this.xMin = aPoint.xValue;
      }

      this.adjustYMinMaxForPoint(aPoint, anIndex);
  },

  /* This method determines if the passed point holds a new min or max in
   * the y axis.
   */
  adjustYMinMaxForPoint: function(aPoint, anIndex)
  {
    if ( aPoint.yValue > this.yMax[anIndex] ) 
    {
        //alert("Scale.adjustYMinMaxForPoint: new y max: (" + aPoint.xValue + "," + aPoint.yValue + 
        //        ") old: ("  + this.yMax + ")");
        this.yMax[anIndex] = aPoint.yValue;
    }
    
    if ( aPoint.yValue < this.yMin[anIndex] ) 
    {
        //alert("Scale.adjustYMinMaxForPoint: new y min: (" + aPoint.xValue + "," + aPoint.yValue + 
        //        ") old: ("  + this.yMin + ")");
        this.yMin[anIndex] = aPoint.yValue;
    }
  },
  
  getXMin: function(anIndex)
  {
    if ( this.key != SCALE_AUTO_KEY )
    {
      // no data
      if ( this.xMin == SCALE_BIGNUM )
        return 0;
    
      // too few points
      if ( this.xMax - this.xMin < SCALE_MIN_X_SPAN )
        return this.xMax - SCALE_MIN_X_SPAN;
    
      return this.xMin;
    }

    // no data
    if ( this.xMinArray[anIndex] == SCALE_BIGNUM )
      return 0;
  
    // too few points
    if ( this.xMaxArray[anIndex] - this.xMinArray[anIndex] < SCALE_MIN_X_SPAN )
      return this.xMaxArray[anIndex] - SCALE_MIN_X_SPAN;
  
    return parseInt(this.xMinArray[anIndex]);

  },
  
  setXMin: function(aValue)
  {
    this.xMin = aValue;
  },
  
  getXMax: function(anIndex)
  {
    if (  this.key != SCALE_AUTO_KEY )
    {
      // no data
      if ( this.xMax == -SCALE_BIGNUM )
      {
        return parseInt(SCALE_MIN_X_SPAN);
      }
    
      return parseInt(this.xMax);
    }

    // no data
    if ( this.xMaxArray[anIndex] == -SCALE_BIGNUM )
    {
      return parseInt(SCALE_MIN_X_SPAN);
    }
  
    return parseInt(this.xMaxArray[anIndex]);
  },
  
  setXMax: function(aValue)
  {
    this.xMax = aValue;
  },
    
  getYMin: function(anIndex)
  {
    return this.yMin[anIndex];
  },
  
  setYMin: function(aValue, anIndex)
  {
    this.yMin[anIndex] = aValue;
  },
  
  getYMax: function(anIndex)
  {
    return this.yMax[anIndex];
  },
  
  setYMax: function(aValue, anIndex)
  {
    this.yMax[anIndex] = aValue;
  }
}
